import re

PATTERNS = {
    "emoji": re.compile(r"<(?:emoji-id|em):(\d+)>(.*?)</(?:emoji-id|em)>", re.DOTALL),
    "mention": re.compile(r"<(?:mention|mo):(\d+)>(.*?)</(?:mention|mo)>", re.DOTALL),
    "link": re.compile(r"<(?:link|url)(?::([^>]+))?>(.*?)</(?:link|url)>", re.DOTALL),
    "spoiler": re.compile(r"<(?:spoiler|sp)>(.*?)</(?:spoiler|sp)>", re.DOTALL),
    "blockquote": re.compile(
        r"<(?:blockquote|bq)(?: hide)?>(.*?)</(?:blockquote|bq)>", re.DOTALL
    ),
}



from telethon.tl.types import (
    MessageEntityBlockquote,
    MessageEntityCustomEmoji,
    MessageEntityMention,
    MessageEntityMentionName,
    MessageEntitySpoiler,
    MessageEntityTextUrl,
)

from .patterns import PATTERNS


class EntitiesManager:
    async def parse(self, text: str):
        matches = []

        for name, pattern in PATTERNS.items():
            for match in pattern.finditer(text):
                matches.append({"name": name, "match": match})

        matches.sort(key=lambda x: x["match"].start())

        clean_text = text
        entities = []
        offset = 0

        for item in matches:
            name = item["name"]
            match = item["match"]
            start = match.start() - offset

            entity = None

            if name == "emoji":
                replacement = match.group(2)
                entity = MessageEntityCustomEmoji(
                    offset=start,
                    length=len(replacement),
                    document_id=int(match.group(1)),
                )
            elif name == "mention":
                user_id = match.group(1)
                replacement = match.group(2)
                entity = MessageEntityMentionName(
                    offset=start, length=len(replacement), user_id=int(user_id)
                )
            elif name == "link":
                url = match.group(1)
                replacement = match.group(2)
                entity = MessageEntityTextUrl(
                    offset=start, length=len(replacement), url=url or replacement
                )
            elif name == "spoiler":
                replacement = match.group(1)
                entity = MessageEntitySpoiler(offset=start, length=len(replacement))
            elif name == "blockquote":
                replacement = match.group(1)
                entity = MessageEntityBlockquote(
                    offset=start,
                    length=len(replacement),
                    collapsed="hide" in match.group(0),
                )

            if entity:
                entities.append(entity)

            clean_text = (
                clean_text[:start] + replacement + clean_text[match.end() - offset :]
            )
            offset += match.end() - match.start() - len(replacement)

        return clean_text, entities

    async def remove(self, text: str):
        clean_text, _ = await self.parse(text)
        return clean_text



from telethon import TelegramClient as BaseTelegramClient
from .session import SessionManager
from .parsing.manager import EntitiesManager


class TelegramClient(BaseTelegramClient):
    def __init__(
        self,
        session,
        api_id,
        api_hash,
        *,
        device_model=None,
        system_version=None,
        app_version=None,
        lang_code="en",
        system_lang_code="en",
        **kwargs,
    ):
        super().__init__(session, api_id, api_hash, **kwargs)
        self._session_manager = SessionManager(self.session)
        self._entities_manager = EntitiesManager()

        self._device_model = device_model
        self._system_version = system_version
        self._app_version = app_version
        self._lang_code = lang_code
        self._system_lang_code = system_lang_code

    async def connect(self):
        self._session_manager.init_session(
            device_model=self._device_model,
            system_version=self._system_version,
            app_version=self._app_version,
            lang_code=self._lang_code,
            system_lang_code=self._system_lang_code,
        )
        await super().connect()

    async def _process_entities(self, text, formatting_entities, parse_mode):
        if not text:
            return text, formatting_entities, parse_mode

        text, new_entities = await self._entities_manager.parse(text)
        if new_entities:
            formatting_entities = list(formatting_entities or []) + new_entities
            formatting_entities.sort(key=lambda e: e.offset)
            parse_mode = None

        return text, formatting_entities, parse_mode

    async def send_message(
        self,
        entity,
        message,
        *,
        parse_mode=(),
        formatting_entities=None,
        **kwargs,
    ):
        message, formatting_entities, parse_mode = await self._process_entities(
            message, formatting_entities, parse_mode
        )
        return await super().send_message(
            entity,
            message,
            parse_mode=parse_mode,
            formatting_entities=formatting_entities,
            **kwargs,
        )

    async def edit_message(
        self,
        entity,
        message=None,
        *,
        parse_mode=(),
        formatting_entities=None,
        **kwargs,
    ):
        message, formatting_entities, parse_mode = await self._process_entities(
            message, formatting_entities, parse_mode
        )
        return await super().edit_message(
            entity,
            message,
            parse_mode=parse_mode,
            formatting_entities=formatting_entities,
            **kwargs,
        )

    async def send_file(
        self,
        entity,
        file,
        *,
        caption=None,
        parse_mode=(),
        formatting_entities=None,
        **kwargs,
    ):
        caption, formatting_entities, parse_mode = await self._process_entities(
            caption, formatting_entities, parse_mode
        )
        return await super().send_file(
            entity,
            file,
            caption=caption,
            parse_mode=parse_mode,
            formatting_entities=formatting_entities,
            **kwargs,
        )

    async def parse_telecats(self, text: str, remove_only=False):
        if remove_only:
            return await self._entities_manager.remove(text)
        return await self._entities_manager.parse(text)



import secrets
from mimesis import Text


class SessionManager:
    def __init__(self, session):
        self.session = session
        self.text_generator = Text("en")

    def _generate_device_model(self):
        return f"{self.text_generator.word().capitalize()} {self.text_generator.word().capitalize()}"

    def _generate_system_version(self):
        return f"{self.text_generator.word().capitalize()} {secrets.randbelow(10)}.{secrets.randbelow(10)}.{secrets.randbelow(100)}"

    def _generate_app_version(self):
        return f"{self.text_generator.word().capitalize()} {self.text_generator.word().capitalize()} {secrets.randbelow(10)}.{secrets.randbelow(10)}"

    def init_session(
        self,
        device_model=None,
        system_version=None,
        app_version=None,
        lang_code="en",
        system_lang_code="en",
    ):
        self.session.device_model = (
            device_model
            or getattr(self.session, "device_model", None)
            or self._generate_device_model()
        )
        self.session.system_version = (
            system_version
            or getattr(self.session, "system_version", None)
            or self._generate_system_version()
        )
        self.session.app_version = (
            app_version
            or getattr(self.session, "app_version", None)
            or self._generate_app_version()
        )
        self.session.lang_code = lang_code
        self.session.system_lang_code = system_lang_code
        self.session.save()
